package beautiful05.step3;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.ozacc.mail.Mail;
import com.ozacc.mail.SendMail;

public abstract class SendMailer<K, E> {

	private static final Log LOG = LogFactory.getLog(SendMailer.class);

	private SendMail sendMail;

	public void setSendMail(SendMail sendMail) {
		this.sendMail = sendMail;
	}

	public void send(String from, String subject, String body) {
		List<K> targetIdList = getTargetIdList();
		for (K targetId : targetIdList) {
			E entity = getEntity(targetId);
			send(entity, from, subject, body);
		}
	}

	private void send(E entity, String from, String subject, String body) {
		try {
			Mail mail = makeMail(entity, from, subject, body);
			if (LOG.isDebugEnabled()) {
				LOG.debug("[𑗐M܂B : mail=" + mail);
			}
			sendMail.send(mail);
			if (LOG.isDebugEnabled()) {
				LOG.debug("[𑗐M܂B : mail=" + mail);
			}
		} catch (Exception ex) {
			LOG.error(ex.getMessage(), ex);
		}
	}

	abstract protected List<K> getTargetIdList();
	abstract protected E getEntity(K targetId);
	abstract protected Mail makeMail(E entity, String from, String subject, String body);
}
